#include <iostream>
#include <fstream>
#include "Challenges.pb.h"
using namespace std;

/* This file shows how to parse and read .challenge and .instance files.
 * Given the path to a .challenge file (and optionally a .instance file associated
 * with the challenge), it prints the contents of the parsed message. It also
 * demonstrates how to access fields in the parsed message.
 *
 * You will need to install the Google protocol buffer library for C++ to
 * build and run this code.
 *
 * More information about parsing can be found at
 * https://developers.google.com/protocol-buffers/docs/reference/cpp-generated
 */

int main(int argc, char* argv[]) {
  // Verify that the version of the library that we linked against is
  // compatible with the version of the headers we compiled against.
  GOOGLE_PROTOBUF_VERIFY_VERSION;

  if (argc < 2 || argc > 3) {
    cerr << "Usage:  " << argv[0] << " path/to/.challenge [path/to/.instance]" << endl;
    return -1;
  }

  Challenge chall;
  // Read the challenge.
  fstream input(argv[1], ios::in | ios::binary);
  if (!chall.ParseFromIstream(&input)) {
    cerr << "Failed to parse challenge." << endl;
    return -1;
  }

  //print the entire challenge
  cout << chall.google::protobuf::Message::DebugString() << endl;

  // example member access

  // number of instances in the challenge
  // (note the change in case compared to the proto file)
  cout << chall.numinstances() << endl;
  // shows which type of parameter the message contains
  if (chall.has_cparams()) {
    cout << "CPARAMS" << endl;
    // cyclotomic index of a challenge
    cout << chall.cparams().m() << endl;
  }
  else if (chall.has_dparams()) {
    cout << "DPARAMS" << endl;
    // number of samples in each instance
    // (note the change in case compared to the proto file)
    cout << chall.dparams().numsamples() << endl;
  }
  else if(chall.has_rparams()) {
    cout << "RPARAMS" << endl;
    // input modulus of the RLWR challenge
    cout << chall.rparams().q() << endl;
  }

  // if the path to a .instance file was supplied:
  if (argc == 3) {
    if (chall.has_cparams()) {
      // parse a InstanceCont
      InstanceCont inst;

      // Read the instance.
      fstream input(argv[2], ios::in | ios::binary);
      if (!inst.ParseFromIstream(&input)) {
        cerr << "Failed to parse instance." << endl;
        return -1;
      }

      // print all contents
      cout << inst.google::protobuf::Message::DebugString() << endl;

      // it is easy to access any member of the parsed message
      /* each instance has many samples (print something from the first sample)
       * each sample is a pair (a,b=a*s+e) (print something in the ring element 'a')
       * each ring element is represented as a list of coefficients with resepct to
       * the decoding basis. (print the first coefficient)
       */
      cout << inst.samples(0).a().xs(0) << endl;
    }
    else if (chall.has_dparams()) {
      // ditto for InstanceDisc
      InstanceDisc inst;
      fstream input(argv[2], ios::in | ios::binary);
      if (!inst.ParseFromIstream(&input)) {
        cerr << "Failed to parse instance." << endl;
        return -1;
      }
      cout << inst.google::protobuf::Message::DebugString() << endl;
      cout << inst.samples(0).a().xs(0) << endl;
    }
    else if(chall.has_rparams()) {
      // ditto for InstanceRLWR
      InstanceRLWR inst;
      fstream input(argv[2], ios::in | ios::binary);
      if (!inst.ParseFromIstream(&input)) {
        cerr << "Failed to parse instance." << endl;
        return -1;
      }
      cout << inst.google::protobuf::Message::DebugString() << endl;
      cout << inst.samples(0).a().xs(0) << endl;
    }
  }

  cout << "See source code in ChallInstParser.cc for examples of how to access specific message members." << endl;

  // Optional:  Delete all global objects allocated by libprotobuf.
  google::protobuf::ShutdownProtobufLibrary();

  return 0;
}